/*
Sample code for test RM1 with the I2C mode based on Arduino UNO!
Connection:
RM1       Arduino
Vcc   ->  5V
SDA   ->  SDA(A4)    pull-up with 4.7K ohm
SCL   ->  SCL(A5)    pull-up with 4.7K ohm
GND   ->  GND
*/

#define Version 180117

/*
This version test one sensor in single sensor mode or test two, three, four or five sensors in multi sensor mode.

|A3|A2|A1|
|--|--|--| single sensor
|--|--|Gd| two sensors
|--|Gd|--| three sensors
|--|Gd|Gd| four sensors
|Gd|--|--| five sensors
-- :open, Gd :Connect to GND

You can start/stop reading from sensor(s) by sending 'S' command via serial.

You can send commands and arguments to sensor(s) via serial with following procedure.
For sending command to sensor, put target I2C address in hexadecimal code followed by command and parameters to Serial port.
Ex. 09I0B sends I command then send argument of 0x0B to the sensor 0x09.
Transmission is terminated by "\n" and must be single command at a time.
*/

//Use I2C address as follows
// The default address specified in the datasheet is 18 (0x12)
// but i2c adressing uses the high 7 bits so it's 9
#define I2CADR 0x09
// second i2c adress is 11
#define I2CADR2 0x0B
// third i2c adress is 12
#define I2CADR3 0x0C
// fourth i2c adress is 13
#define I2CADR4 0x0D
// fifth i2c adress is 14
#define I2CADR5 0x0E

#include <Wire.h>
#include <stdio.h>

byte mode ;
byte Run = 0x01 ;

byte Hascii2bin(byte input)
{
  //if the bit6 is 1 then 
  if ( input >= 0x40 )
  {
    //it must be 0x0A - 0x0F
    input += 9 ;
  }
  input &= 0x0F ;
  return input ;
}

void ViewSensor(unsigned int I2Cadress)
{
  Wire.requestFrom(I2Cadress, 4);    // request 4 bytes from slave sensor1
  // receive reading from sensor
  if (2 <= Wire.available())   // if two bytes were received
  {
    byte Id = Wire.read();  //receive indication code
    unsigned int reading = Wire.read();  // receive high byte (overwrites previous reading)
    reading = reading << 8;    // shift high byte to be high 8 bits
    reading |= Wire.read(); // receive low byte as lower 8 bits
    byte SensLevel = Wire.read();  // receive sensing level
    char string[] = "           ";
    sprintf (string, "%c %dmm %2X", Id, reading, SensLevel) ;
    Serial.print(string);   // print the reading
    Serial.print(",  ");
  }
}

void setup()
{
  //Initialize test mode pin
  pinMode (A1, INPUT);
  pinMode (A2, INPUT);
  pinMode (A3, INPUT);
  digitalWrite(A1, HIGH);  // set pullup on analog pin 1
  digitalWrite(A2, HIGH);  // set pullup on analog pin 2
  digitalWrite(A3, HIGH);  // set pullup on analog pin 3
  delay(100);             // wait a bit for pullupped pin get stable

  Wire.begin();                // join i2c bus (address optional for master)
  Serial.begin(9600);          // start serial communication at 9600bps
  Serial.print("I2Ctester");
  Serial.println(Version);

  mode = 0 ;
  if ( digitalRead(A1) == 0 ) { mode |= 0x01 ;}
  if ( digitalRead(A2) == 0 ) { mode |= 0x02 ;}
  if ( digitalRead(A3) == 0 ) { mode |= 0x04 ;}
}

void loop()
{
  if ( Run == 0x01 )
  {
    Serial.print("sensor1");
    if ( mode != 0 )
    {
      Serial.print(", sensor2");
    }
    if ( mode >= 2 )
    {
      Serial.print(", sensor3");
    }
    if ( mode >= 3 )
    {
      Serial.print(", sensor4");
    }
    if ( mode >= 4 )
    {
      Serial.print(", sensor5");
    }
    Serial.println();
  
    // request reading from sensor1
    unsigned int I2Cadress = I2CADR ;// request 4 bytes from slave sensor1
    ViewSensor(I2Cadress) ;
  
    //if two, three, four or five sensor mode
    if ( mode != 0 )
    {
      // request reading from sensor2
      unsigned int I2Cadress = I2CADR2 ;// request 4 bytes from slave sensor2
      ViewSensor(I2Cadress) ;
    }
    //if three, four or five sensor mode
    if ( mode >= 2 )
    {
      // request reading from sensor3
      unsigned int I2Cadress = I2CADR3 ;// request 4 bytes from slave sensor3
      ViewSensor(I2Cadress) ;
    }
    //if four or five sensor mode
    if ( mode >= 3 )
    {
      // request reading from sensor4
      unsigned int I2Cadress = I2CADR4 ;// request 4 bytes from slave sensor4
      ViewSensor(I2Cadress) ;
    }
    //if five sensor mode
    if ( mode >= 4 )
    {
      // request reading from sensor5
      unsigned int I2Cadress = I2CADR5 ;// request 4 bytes from slave sensor5
      ViewSensor(I2Cadress) ;
    }
    Serial.println();
  }

  // if some command comes from serial then parse it and transfer the arguments to I2C
  if (Serial.available())
  {
    char c = Serial.read();
    if ( c == 'S' )
    {
      //reverse Run flag
      Run ^= 0x01 ;
      //Skip to \n
      while ( 1 )
      {
        while (! Serial.available()){}
        c = Serial.read();
        if ( c == '\n' ) { break ;}
      }
    }
    else
    {
      byte i = Hascii2bin(c) ;
      i <<= 4 ;
      while (! Serial.available()){}
      c = Serial.read();
      i += Hascii2bin(c) ;
  
      //read command
      while (! Serial.available()){}
      char com = Serial.read();
  
      byte param ;
      byte paramflag = 0 ;
      while ( 1 )
      {
        while (! Serial.available()){}
        c = Serial.read();
        if ( c == '\n' ) { break ;}
        //Cancel CR code
        if ( c != '\r' )
        {
          paramflag = 1 ;
          param <<= 4 ;
          param += Hascii2bin(c) ;
        }
      }
  
      Wire.beginTransmission(i);
      //send command
      Wire.write(com);
      //if there is some parameter
      if ( param != 0xF0 )
      {
        Wire.write(param);
      }
      Wire.endTransmission();
  
      // print the parsed reading
      Serial.println();
      char string[] = "           ";
      if ( paramflag == 1 )
      {
        sprintf ( string, "I2C(%02X) %c%02X", i, com, param ) ;
      }
      else
      {
        sprintf ( string, "I2C(%02X) %c", i, com ) ;
      }
      Serial.println(string);
    }
  }
  delay(500);                  // wait a bit since people have to read the output :)
}